const express = require('express');
const moment = require('moment');
const connection = require('./db');
const languageMessages = require('./language_messages');
const {fetchUserData} = require('./function_app');
// const { error } = require('console');

const AddAvatar = (req, res) => {
    try {
        const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");
        const { avatar_id, user_id, description } = req.body;

        if (!user_id || !avatar_id ) {
            return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
        }

        const sql = `UPDATE user_master SET avatar_id = ?, updatetime = ?, about = ?,profile_complete=1 WHERE user_id = ? AND delete_flag = 0`;
        const values = [avatar_id, updatetime, description, user_id];
       
        // return res.status(200).json({ success: true, msg: languageMessages.AvatarAdded, data: user_id });

        connection.query(sql, values, (err, info) => {
            if (err) {
                console.error("Error updating avatar:", err);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError });

                
            } else {
                fetchUserData(user_id, (error, userDataArray) => {
                    if (error) {
                        console.error('Error fetching user data:', error);
                        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                    }
                    return res.status(200).json({ success: true, msg: languageMessages.AvatarAdded, data: userDataArray });   
                });
                
                

            }
        });
    } catch (error) {
        console.error("Error in AddAvatar function:", error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
};

module.exports = { AddAvatar };
