const express = require('express');
const multer = require('multer');
const path = require('path');
const moment = require('moment');
const connection = require('./db');
const languageMessages = require('./language_messages');
const compressing = require('compressing');
const {fetchUserData,getCurrentTimeInTimeZone} = require("./function_app");
const {currentTime} = require("./time");


const storage = multer.diskStorage({

    destination: function (req, file, cb) {

      cb(null, './webservice/images/'); // Save uploaded files to the 'webservice/images' directory

    },

    filename: function (req, file, cb) {

      const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);

      cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));

    }

  });

  
const upload = multer({ storage: storage });

// create post start

const CreatePost = async (req, res) => {
    try {
        // const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");
        // const updatetime = getCurrentTimeInTimeZone();
        const files = req.files;
        const { user_id, post_type, caption, keyword_ids, location, latitude, longitude, keyword } = req.body;

        if (!user_id || !location || !latitude || !longitude) {
            return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
        }

        const currentTimeAmerica =  moment.tz('America/New_York').format('YYYY-MM-DD HH:mm:ss')
        var account_fund_balance = 0.00;
        var currect_location_permanent = 0;
        var current_location = "";
        var current_lattitude = 0;
        var current_longitude = 0;
        var account_currency = 0;

        var sql1 = "SELECT user_id,active_flag,currect_location_permanent,current_location,current_lattitude,current_longitude,account_currency,account_fund_balance from user_master where delete_flag = 0 AND user_id = ?";
        var values1 = [user_id];

        const information = await new Promise((resolve, reject) => {
            connection.query(sql1, values1, (err, information) => {
                if (err) {
                    reject(err);
                } else {
                    resolve(information);
                }
            });
        });

        if (information.length <= 0) {
            return res.status(200).json({ success: false, msg: languageMessages.msgUserNotFound , data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }

        currect_location_permanent = information[0].currect_location_permanent;
        current_location = information[0].current_location;
        current_lattitude = information[0].current_lattitude;
        current_longitude = information[0].current_longitude;
        account_fund_balance = information[0].account_fund_balance;
        account_currency = information[0].account_currency;

        if (post_type == 1) {
            const adminPostPriceQuery = "SELECT top_post_fee_usd, top_post_fee_yuan FROM admin_manage_master WHERE delete_flag = 0";

            const adminPostPriceResult = await new Promise((resolve, reject) => {
                connection.query(adminPostPriceQuery, (adminPostPriceError, adminPostPriceResult) => {
                    if (adminPostPriceError) {
                        reject(adminPostPriceError);
                    } else {
                        resolve(adminPostPriceResult);
                    }
                });
            });

            const adminPostPrice = adminPostPriceResult[0];
            const fee = account_currency === 'USD' ? adminPostPrice.top_post_fee_usd : adminPostPrice.top_post_fee_yuan;
            const currencySymbol = account_currency === 'USD' ? '$' : '¥';

            if (parseFloat(account_fund_balance) < parseFloat(fee)) {
                return res.status(200).json({ success: false, msg: languageMessages.insufficientFund });
            }

            const newBalance = account_fund_balance - fee;
            const updateBalQuery = "UPDATE user_master SET account_fund_balance = ?, updatetime = ? WHERE delete_flag = 0 AND user_id = ?";

            await new Promise((resolve, reject) => {
                connection.query(updateBalQuery, [newBalance,currentTime, user_id], (updateBalError, updateBalResult) => {
                    if (updateBalError) {
                        reject(updateBalError);
                    } else {
                        resolve(updateBalResult);
                    }
                });
            });
        }

        const updateUser = "UPDATE user_master SET address=?,latitude=?,longitude=?,updatetime=? WHERE delete_flag=0 and user_id=?";
        await new Promise((resolve, reject) => {
            connection.query(updateUser, [location, latitude, longitude,currentTime, user_id], (updateUserError, updateUserResult) => {
                if (updateUserError) {
                    reject(updateUserError);
                } else {
                    resolve(updateUserResult);
                }
            });
        });

        const insertPostQuery = "INSERT INTO post_master (`post_type`, `user_id`, `caption`, `keyword_ids`, `location`, `latitude`, `longitude`, `createtime`, `updatetime`,`keyword`) VALUES (?,?,?,?,?,?,?,?,?,?)";
        const values = currect_location_permanent === 0 ? [post_type, user_id, caption, keyword_ids, location, latitude, longitude,currentTimeAmerica,currentTimeAmerica, keyword] : [post_type, user_id, caption, keyword_ids, current_location, current_lattitude, current_longitude,currentTimeAmerica,currentTimeAmerica, keyword];

        const postResults = await new Promise((resolve, reject) => {
            connection.query(insertPostQuery, values, (postError, postResults) => {
                if (postError) {
                    reject(postError);
                } else {
                    resolve(postResults);
                }
            });
        });

        const post_id = postResults.insertId;

        if (files.length > 0) {
            for (const file of files) {
                const sql = 'INSERT INTO post_image_master (post_id, image, createtime, updatetime) VALUES (?, ?,?,?)';
                const values = [post_id, file.filename,currentTime,currentTime];
                await new Promise((resolve, reject) => {
                    connection.query(sql, values, (error, results) => {
                        if (error) {
                            reject(error);
                        } else {
                            resolve(results);
                        }
                    });
                });
            }
        }
        
        return res.status(200).json({ success: true, msg: languageMessages.postCreatedSuccessfully });
    } catch (err) {
        console.error("Internal server error:", err);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError , key: 1 });
    }
}


// create post end

// update post start

const updatePost = async (req,res)=>{

    try {

      
        const files = req.files;
        // console.log(files);
        const { user_id, post_type, caption, keyword_ids, location, latitude, longitude,keyword,post_id,delete_id} = req.body;

        if (!user_id || !location || !latitude || !longitude || !post_id) {
            return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
        }

        const updatetime = getCurrentTimeInTimeZone();
      // check user exists or not start

      var sql1 = "SELECT user_id from user_master where delete_flag = 0 AND user_id = ?";
      var values1 = [user_id];
      connection.query(sql1, values1, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }

        });
       
         // check user exists or not end

         const updateQuery = "UPDATE post_master SET post_type =? ,caption=?,keyword_ids=?, `keyword`=?, `updatetime` = ? WHERE delete_flag=0 and user_id =? and post_id = ? ";
         const updateValues = [post_type,caption,keyword_ids,keyword,currentTime,user_id,post_id];
         connection.query(updateQuery,updateValues,(updateError,UpdateResult)=>{
            if(updateError){
                return res.status(500).json({ success: false, msg: languageMessages.postUpdateUnsuccessfully,updateError });
            }

         });

         // old images delete start
        
        var post_delete_id = delete_id.split(",");
        for(var data of post_delete_id){
           deleteImageQuery = "DELETE FROM post_image_master WHERE post_image_id = ?";
           deleteQueryValues = [data];
           connection.query(deleteImageQuery,deleteQueryValues,(deleteError,deleteResult)=>{
               if(deleteError){
                   return res.status(500).json({ success: false, msg: languageMessages.postUpdateUnsuccessfully,deleteError});
               }
           });
        }
        // old images delete end

        // insert new images start

        if (files.length > 0) {
            files.forEach(file => {
              const sqlImage = 'INSERT INTO post_image_master (post_id, image, createtime, updatetime) VALUES (?, ?,?,?)';
              const valuesImage = [post_id, file.filename,currentTime,currentTime];
          
              connection.query(sqlImage, valuesImage, (error, results) => {
                if (error) {
                  console.error('Error inserting data into MySQL: ' + error);
                //   return res.status(500).send('Error inserting data into MySQL');
                }  
              });
            });
          }
            return res.status(200).json({ success: true, msg: languageMessages.postUpdateSuccessfully });

        // insert new images end
        
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError,error});
    }

}

// update post end

// delete post start

const DeletePost = async (req,res)=>{
  
    try {
        
        // const files = req.files;
        // console.log(files);
        const { user_id,post_id} = req.body;

        if (!user_id || !post_id) {
            return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
        }

        const updatetime = getCurrentTimeInTimeZone();

         // check user exists or not start

      var sql1 = "SELECT user_id from user_master where delete_flag = 0 AND user_id = ?";
      var values1 = [user_id];
      connection.query(sql1, values1, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        });
       
         // check user exists or not end

         // delete the post start

         const updateQuery = "UPDATE post_master SET delete_flag = 1,`updatetime` = ? WHERE delete_flag=0 and user_id =? and post_id = ? ";
         const updateValues = [currentTime,user_id,post_id];
         connection.query(updateQuery,updateValues,(updateError,UpdateResult)=>{
            if(updateError){
                return res.status(500).json({ success: false, msg: languageMessages.postDeleteUnsuccessfully,updateError });
            }

         });

         // delete the post end

         return res.status(200).json({ success: true, msg: languageMessages.postDeleteSuccessfully});

    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError,error});
    }
}

// delete post end

// Function to format the date
function formatDate(createtime) {
    const date = new Date(createtime);
    const options = { year: 'numeric', month: 'short', day: '2-digit' };
    return date.toLocaleDateString('en-US', options);
}


// getting all post images start
function GetPostImages(post_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT post_image_id,image FROM post_image_master WHERE post_id = ? and delete_flag=0`;
        connection.query(query, [post_id], (error, results) => {
            if (error) {
                reject(error);
                return;
            }

            let post_image_arr = [];
            if (results.length > 0) {
                post_image_arr = results.map(row => ({
                    post_image_id: row.post_image_id,
                    image: row.image,
                    status : 0,
                   
                }));
                resolve(post_image_arr);
            } else {
                resolve('NA');
            }
        });
    });
}
// getting all post images end


// get all my post start
const GetMyPost = async (req, res) => {

    const { user_id,longitude,latitude,language_id} = req.query; // Retrieve user_id from query parameters

    try {
       
        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

         // check user exists or not start

         var about = 'NA';
      var sql = "SELECT user_id,active_flag,about from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }
            about = information[0].about;
        });

        // check user exists or not end

        //  var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;
     
        var query = "SELECT pm.post_id, pm.post_type, pm.caption, pm.keyword_ids, pm.location, pm.latitude, pm.longitude, pm.createtime, pm.keyword, um.username, um.avatar_id, um.user_id FROM post_master AS pm INNER JOIN user_master AS um ON um.user_id = pm.user_id WHERE pm.delete_flag = 0 AND pm.user_id = ? ORDER BY pm.post_id DESC";
     

        connection.query(query, [user_id], async (err, rows) => {
            if (err) {
                console.error('Error executing MySQL query:', err);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
            }
            var localPostArr = [];
            var topPostArr = [];

            if (rows.length > 0) {
                for (const row of rows) {
                    const post_images = await GetPostImages(row.post_id);
                    var post_images_cnt = post_images.length;
                    const like_status = await likePostStatus(row.post_id,user_id);
                    const dislike_status = await dislikePostStatus(row.post_id,user_id);
                    const likeCount = await likePostCount(row.post_id);
                    const dislikeCount = await dislikePostCount(row.post_id);
                    const avatar_image = await GetAvatar(user_id);
                    const post_time = calculateTimeElapsed(row.createtime);
                    const post_time_chinese = calculateTimeElapsedChinese(row.createtime);
                    const saveUnsaveStatus = await saveUnsaveStatus1(row.post_id,user_id);
                    let keywords_ids =  row.keyword_ids;
                    let keyword =  row.keyword;
                     if (keywords_ids === "" || keywords_ids === null || keyword == null ) {
                            keywords_ids = 'NA';  
                            keyword = 'NA';       
                        }

 let distance = getDistance(row.latitude, row.longitude, latitude, longitude);
                        distance = distance? Math.floor(distance):1;
                    // if(row.post_type === 0){
                        localPostArr.push({
                            'username': row.username,
                            'avatar_id': row.avatar_id,
                            'user_id' : row.user_id,
                            'avatar_image' : avatar_image,
                            'post_id': row.post_id,
                            'post_image': post_images,
                            'post_type': row.post_type,
                            'caption': row.caption,
                            'keyword_ids': keywords_ids,
                            'keyword': keyword,
                            'post_images_cnt': post_images_cnt,
                            'like_status': like_status,
                            'dislike_status': dislike_status,
                            'likeCount': likeCount,
                            'distance': distance,
                            'dislikeCount': dislikeCount,
                            'location': row.location,
                            'latitude': row.latitude,
                            'longitude': row.longitude,
                            'createtime': row.createtime,
                            'createtime_format': formatDate(row.createtime), // Format createtime
                            'post_time':language_id==0? post_time:post_time_chinese,
                            'saveUnsaveStatus':saveUnsaveStatus,
                            'about':about,
                        });
                    // }
                    if(row.post_type === 1){
                        topPostArr.push({
                            'username': row.username,
                            'avatar_id': row.avatar_id,
                            'user_id' : row.user_id,
                            'avatar_image' : avatar_image,
                            'post_id': row.post_id,
                            'post_image': post_images,
                            'post_type': row.post_type,
                            'caption': row.caption,
                            'keyword_ids': keywords_ids,
                            'keyword': keyword,
                            'post_images_cnt': post_images_cnt,
                            'like_status': like_status,
                            'dislike_status': dislike_status,
                            'likeCount': likeCount,
                            'distance': distance,
                            'dislikeCount': dislikeCount,
                            'location': row.location,
                            'latitude': row.latitude,
                            'longitude': row.longitude,
                            'createtime': row.createtime,
                            'createtime_format': formatDate(row.createtime), // Format createtime
                            'post_time':language_id==0? post_time:post_time_chinese,
                            'saveUnsaveStatus':saveUnsaveStatus,
                            'about':about,
                        });
                    }
                    
                }
            } 
            if(localPostArr.length === 0){
                var localPostArr = 'NA';
            }
            if(topPostArr.length === 0){
                var topPostArr = 'NA';
            }

            var totalPostCount = await totalLocalpost(user_id);
            var totalTopPostCount = await totalToppost(user_id);
            var likeByUser = await likecount(user_id)
            const record = { success: true, msg: languageMessages.msgDataFound, localPostArr: localPostArr,topPostArr:topPostArr,totalPostCount:totalPostCount,totalTopPostCount:totalTopPostCount,likeByUser:likeByUser};
            res.json(record);
        });
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2 });
    }
};
// get all my post start


const postLike = async (req, res) => {
    const { user_id, post_id } = req.body;

    try {
        if (!user_id || !post_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        // Check if user exists and is active
        const userQuery = "SELECT user_id, active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
        const userValues = [user_id];
        connection.query(userQuery, userValues, async (userErr, userRows) => {
            if (userErr) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if (userRows.length <= 0) {
                return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }
            const isActive = userRows[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }

            // Check if user already liked the post
            const selectLikeQuery = "SELECT post_like_id, type FROM post_like_master WHERE user_id=? AND post_id=? AND type = 0";
            const selectLikeValues = [user_id, post_id];
            await connection.query(selectLikeQuery, selectLikeValues, async (likeErr, likeRows) => {
                if (likeErr) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: 1 });
                }
                if (likeRows.length > 0) {
                    // User already liked the post, delete like
                    const deleteLikeQuery = "DELETE FROM post_like_master WHERE user_id = ? AND post_id = ? AND type = 0";
                    const deleteLikeValues = [user_id, post_id];
                    await connection.query(deleteLikeQuery, deleteLikeValues, (deleteErr, deleteResult) => {
                        if (deleteErr) {
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: 2 });
                        }
                        return res.status(200).json({ success: true, msg: languageMessages.UnlikeSuccessfully });
                    });
                } else {
                    // User didn't like the post, insert like
                    const insertLikeQuery = "INSERT INTO post_like_master (`post_id`, `user_id`, `createtime`, `updatetime`) VALUES (?,?,?,?)";
                    const insertLikeValues = [post_id, user_id,currentTime,currentTime];
                    await connection.query(insertLikeQuery, insertLikeValues, async (insertErr, insertResult) => {
                        if (insertErr) {
                            return res.status(500).json({ success: false, msg: languageMessages.likeUnsuccessfully, error: insertErr });
                        }

                        // if he is dislike then remove

                        await connection.query("DELETE FROM post_like_master WHERE user_id = ? AND post_id = ? AND type = 1",[user_id,post_id],(dislikeErr,dislikeResult)=>{});

                        // if he is dislike then remove

                        // Check if there are enough dislikes to convert post
                        const selectAdminDislikeQuery = "SELECT post_like_count FROM admin_manage_master WHERE delete_flag = 0";
                        await connection.query(selectAdminDislikeQuery, async (adminErr, adminResult) => {
                            if (adminErr) {
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: adminErr });
                            }
                            if (adminResult.length > 0) {
                                const postCountForDelete = adminResult[0].post_like_count;
                                const likeCount = await likePostCount(post_id);
                                if (postCountForDelete === likeCount) {
                                    const post_type = 1;
                                    const deleteByDislikeQuery = "UPDATE post_master SET post_type = ?, converted = 1, updatetime = ? WHERE post_id = ?";
                                    await connection.query(deleteByDislikeQuery, [post_type,currentTime,post_id], (deleteByDislikeErr, deleteByDislikeResult) => {
                                        if (deleteByDislikeErr) {
                                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: deleteByDislikeErr });
                                        }
                                        return res.status(200).json({ success: true, msg: languageMessages.likeSuccessfully,postCountForDelete,likeCount});
                                    });
                                }
                            }
                        });
                        return res.status(200).json({ success: true, msg: languageMessages.likeSuccessfully});
                    });
                }
            });
        });
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 10 });
    }
};


// post like end

// post dislike start


const postDislike = async (req, res) => {
    const { user_id, post_id } = req.body;

    try {
        if (!user_id || !post_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        // Check if user exists and is active
        const userQuery = "SELECT user_id, active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
        const userValues = [user_id];
        connection.query(userQuery, userValues, async (userErr, userRows) => {
            if (userErr) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if (userRows.length <= 0) {
                return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }
            const isActive = userRows[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }
            
            // Check if post exists and is active
            const postQuery = "SELECT post_id, post_type,converted FROM post_master WHERE delete_flag = 0 AND post_id = ?";
            const postValues = [post_id];
            connection.query(postQuery, postValues, async (postErr, postRows) => {
                if (postErr) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
                }
                if (postRows.length <= 0) {
                    return res.status(500).json({ success: false, msg: languageMessages.msgPostNotFound, data: {} });
                }
                const post_type_get = postRows[0].post_type ;
                const converted_get = postRows[0].converted ;
                
            

            // Check if user already disliked the post
            const selectDislikeQuery = "SELECT post_like_id FROM post_like_master WHERE user_id=? AND post_id=? AND type = 1";
            const selectDislikeValues = [user_id, post_id];
            connection.query(selectDislikeQuery, selectDislikeValues, async (dislikeErr, dislikeRows) => {
                if (dislikeErr) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: 1 });
                }
                if (dislikeRows.length > 0) {
                    // User already disliked the post, delete dislike
                    const deleteDislikeQuery = "DELETE FROM post_like_master WHERE user_id = ? AND post_id = ? AND type = 1";
                    const deleteDislikeValues = [user_id, post_id];
                    connection.query(deleteDislikeQuery, deleteDislikeValues, (deleteErr, deleteResult) => {
                        if (deleteErr) {
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: 2 });
                        }
                        return res.status(200).json({ success: true, msg: languageMessages.UnDisLikeSuccessfully });
                    });
                } else {
                    // User didn't dislike the post, insert dislike
                    const updatetime = getCurrentTimeInTimeZone();
                    const type = 1;
                    const insertDislikeQuery = "INSERT INTO post_like_master (`post_id`, `user_id`, `createtime`, `updatetime`, `type`) VALUES (?, ?, ?, ?, ?)";
                    const insertDislikeValues = [post_id, user_id, currentTime, currentTime, type];
                    connection.query(insertDislikeQuery, insertDislikeValues, async (insertErr, insertResult) => {
                        if (insertErr) {
                            return res.status(500).json({ success: false, msg: languageMessages.disLikeUnsccessfully, error: insertErr });
                        }

                        await connection.query("DELETE FROM post_like_master WHERE user_id = ? AND post_id = ? AND type = 0",[user_id,post_id],(dislikeErr,dislikeResult)=>{});

                        // Check if there are enough dislikes to delete post
                        const selectAdminDislikeQuery = "SELECT post_dislike_count FROM admin_manage_master WHERE delete_flag = 0";
                        connection.query(selectAdminDislikeQuery, async (adminErr, adminResult) => {
                            if (adminErr) {
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: adminErr });
                            }
                            if (adminResult.length > 0) {
                                
                                const postCountForDelete = adminResult[0].post_dislike_count;
                                const dislikeCount = await dislikePostCount(post_id);
                                if (postCountForDelete === dislikeCount) {
                                    if(post_type_get === 0 && converted_get === 0){
                                        const delete_by = 1;
                                        const deleteByDislikeQuery = "UPDATE post_master SET delete_by = ?, delete_flag = 1 WHERE post_id = ?";
                                        connection.query(deleteByDislikeQuery, [delete_by, post_id], (deleteByDislikeErr, deleteByDislikeResult) => {
                                            if (deleteByDislikeErr) {
                                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: deleteByDislikeErr });
                                            }
                                        });
                                    }
                                }
                            }
                        });

                        return res.status(200).json({ success: true, msg: languageMessages.disLikesccessfully });
                    });
                }
            });
        });
        });
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 10 });
    }
};


// post dislike end


// check like status start
function likePostStatus(post_id, user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT post_like_id FROM post_like_master WHERE post_id = ? AND user_id = ? AND type = 0 AND delete_flag = 0`;
        connection.query(query, [post_id, user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve('Like');
            } else {
                resolve('Unlike');
            }
        });
    });
}
// check like status end

// check dislike status start
function dislikePostStatus(post_id, user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT post_like_id FROM post_like_master WHERE post_id = ? AND user_id = ? AND type = 1 AND delete_flag = 0`;
        connection.query(query, [post_id, user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve('Disike');
            } else {
                resolve('Undislike');
            }
        });
    });
}
// check dislike status end


// check dislike status start
function saveUnsaveStatus1(post_id, user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT save_id FROM save_master WHERE post_id = ? AND user_id = ? AND delete_flag = 0`;
        connection.query(query, [post_id, user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(1);
            } else {
                resolve(0);
            }
        });
    });
}
// check dislike status end

// check like count start
function likePostCount(post_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT count(post_like_id) as like_count FROM post_like_master WHERE post_id = ? AND type = 0 AND delete_flag = 0`;
        connection.query(query, [post_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(results[0].like_count);
            } else {
                resolve(0);
            }
        });
    });
}
// check like count end


// check dislike count start
function dislikePostCount(post_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT count(post_like_id) as like_count FROM post_like_master WHERE post_id = ? AND type = 1 AND delete_flag = 0`;
        connection.query(query, [post_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(results[0].like_count);
            } else {
                resolve(0);
            }
        });
    });
}
// check dislike count end


// localpost count start

function totalLocalpost(user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT count(post_id) as like_count FROM post_master WHERE user_id = ? AND post_type = 0 AND delete_flag = 0`;
        connection.query(query, [user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(results[0].like_count);
            } else {
                resolve(0);
            }
        });
    });
}

// localpost count end

// top post count start
function totalToppost(user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT count(post_id) as like_count FROM post_master WHERE user_id = ? AND post_type = 1 AND delete_flag = 0`;
        connection.query(query, [user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(results[0].like_count);
            } else {
                resolve(0);
            }
        });
    });
}

// top post count end   

// like post count start

function likecount(user_id) {
    return new Promise((resolve, reject) => {
        const query = `SELECT post_id FROM post_master WHERE user_id = ? AND delete_flag = 0`;
        connection.query(query, [user_id], (error, results) => {
            if (error) {
                reject(`Error fetching like status: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                const postIds = results.map(result => result.post_id);
                const promises = postIds.map(post_id => {
                    return new Promise((resolve, reject) => {
                        const sql = `SELECT count(post_like_id) as like_count FROM post_like_master WHERE post_id = ? AND type = 0 AND delete_flag = 0`;
                        connection.query(sql, [post_id], (err, data) => {
                            if (err) {
                                reject(`Error fetching like status: ${err.message}`);
                                return;
                            }
                            if (data.length > 0) {
                                resolve(data[0].like_count);
                            } else {
                                resolve(0);
                            }
                        });
                    });
                });
                Promise.all(promises)
                    .then(likeCounts => {
                        const totalLikes = likeCounts.reduce((total, current) => total + current, 0);
                        resolve(totalLikes);
                    })
                    .catch(err => {
                        reject(`Error fetching like status: ${err.message}`);
                    });
            } else {
                resolve(0);
            }
        });
    });
}

// like post count end


// get home page all post start

const GetAllPost = async (req, res) => {

    const { user_id ,longitude,latitude,filter_type,language_id} = req.query; 

    try {
       
        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

         // check user exists or not start

      var sql = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, err:err});
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }


        
            // get admin local top post cnt start
       
            // let localPostCntSet = 0;
            // let topPostCntSet = 0;

            // const adminSet = ""

            // get admin local top post cnt end

        var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;


            // var query = `SELECT ps.post_id,ps.post_type,ps.caption,ps.keyword_ids,ps.location,ps.latitude,ps.longitude,ps.createtime,ps.keyword,us.username,us.km_miles,us.avatar_id,us.user_id,us.about,us.user_type FROM post_master as ps INNER JOIN user_master as us ON us.user_id = ps.user_id WHERE us.delete_flag = 0 and ps.delete_flag=0 and ps.latitude != '' and ps.longitude != '' order by ps.post_id desc`;
            var query ;
            if(filter_type == 0){
                query= `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != '' and longitude != '' order by createtime asc`;
            }else{
                query = `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != '' and longitude != '' order by distance asc`;
            }
          
            connection.query(query, async (err, rows) => {
                if (err) {
                    console.error('Error executing MySQL query:', err);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
                }
                var localPostArr = [];
                var topPostArr = [];
               
    
                if (rows.length > 0) {
                    for (const row of rows) {
                        const post_images = await GetPostImages(row.post_id);
                        var post_images_cnt = post_images.length;
                        const like_status = await likePostStatus(row.post_id,user_id);
                        const dislike_status = await dislikePostStatus(row.post_id,user_id);
                        const likeCount = await likePostCount(row.post_id);
                        const dislikeCount = await dislikePostCount(row.post_id);
                        const saveUnsaveStatus = await saveUnsaveStatus1(row.post_id,user_id);
                        // const avatar_image = await GetAvatar(row.avatar_id);  
                        const post_time = calculateTimeElapsed(row.createtime);
                        const post_time_chinese = calculateTimeElapsedChinese(row.createtime);
                        if(user_id == row.user_id){
                            var my_post_status = 1;
                        }else{
                            var my_post_status = 0;
                        }
                        let distance = getDistance(row.latitude, row.longitude, latitude, longitude);
                        distance = Math.floor(distance);
                            
                        let keywords_ids =  row.keyword_ids;
                        let keyword =  row.keyword;
                        if (keywords_ids === "" || keywords_ids === null || keyword == null) {
                            keywords_ids = 'NA';  
                            keyword = 'NA';       
                        }

                        // get post user detail data start 

                        let user_type = 1;
                        let about = null;
                        let username = 'NA';
                        let avatar_id = 0;
                        let avatar_image = 'NA';
                        fetchUserData(row.user_id, (error, userDataArray) => {
                            if (error) {
                                console.error('Error fetching user data:', error);
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                            }


                            if(userDataArray != null){
                            user_type = userDataArray.user_type;
                            about = userDataArray.about;
                            username = userDataArray.username;
                            avatar_id = userDataArray.avatar_id;
                            avatar_image = userDataArray.avatar_image;
                            }
                      
                        // get post user detail data end 

                        let caption = row.caption != "" ? row.caption:null;

                        // get my all keywords start 

                    
                        // get my all keywords end

                        if(row.post_type === 0){

                            // if(home_local_count >= count_local){

                            localPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': (caption) ? caption : null ,
                                'keyword_ids':keywords_ids,
                                'keyword': keyword,
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                // 'post_time':post_time,
                                'post_time':language_id==0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        // }
                        }else{
                            // if(home_top_count >= count_top){
                            topPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': (caption) ? caption : null ,
                                'keyword_ids':keywords_ids,
                                'keyword': keyword,
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                // 'post_time':post_time,
                                'post_time':language_id == 0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        // }

                        
                        }
                    });
                    }
                }
    
                if(localPostArr.length === 0){
                    var localPostArr = 'NA';
                }
                if(topPostArr.length === 0){
                    var topPostArr = 'NA';
                }
           
                 
                
                const record = { success: true, msg: languageMessages.msgDataFound, localPostArr: localPostArr,topPostArr:topPostArr};
                res.json(record);
            });
        
        });

        // check user exists or not end
       

       
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2 });
    }
};

// get home page all post end


function GetAvatar(avatar_id) {
    return new Promise((resolve, reject) => {
        const query = 'SELECT avatar_image FROM avatar_master WHERE avatar_id = ? AND delete_flag = 0';
        connection.query(query, [avatar_id], (error, results) => {
            if (error) {
                reject(`Error fetching avatar image: ${error.message}`);
                return;
            }

            if (results.length > 0) {
                resolve(results[0].avatar_image);
            } else {
                resolve('NA');
            }
        });
    });
}


function calculateTimeElapsedChinese(timestamp) {
    const givenTime = new Date(timestamp);
    // const currentTimess = new Date(currentTime);
    const currentTime_new = new Date().toLocaleString("en-US", {timeZone: "America/New_York"});

    const currentTime_nn_new = new Date(currentTime_new);

    const timeDifference = currentTime_nn_new - givenTime;

    const seconds = Math.floor(timeDifference / 1000);
    const minutes = Math.floor(seconds / 60);
    const hours = Math.floor(minutes / 60);
    const days = Math.floor(hours / 24);
    const months = Math.floor(days / 30);
    const years = Math.floor(months / 12);

    if (years > 0) {
        return `${years}年前`;
    } else if (months > 0) {
        return `${months}月前`;
    } else if (days > 0) {
        return `${days}天前`;
    } else if (hours > 0) {
        return `${hours}小时前`;
    } else if (minutes > 0) {
        return `${minutes}分钟前`;
    } else {
        return `${seconds}秒前`;
    }
}


function calculateTimeElapsed(timestamp) {
    const givenTime = new Date(timestamp);
  
    const currentTime_n = new Date().toLocaleString("en-US", {timeZone: "America/New_York"});

    const currentTime_nn = new Date(currentTime_n);
   
    const timeDifference = currentTime_nn - givenTime;

    const seconds = Math.floor(timeDifference / 1000);
    const minutes = Math.floor(seconds / 60);
    const hours = Math.floor(minutes / 60);
    const days = Math.floor(hours / 24);
    const months = Math.floor(days / 30);
    const years = Math.floor(months / 12);

    if (years > 0) {
        return `${years} year${years > 1 ? 's' : ''} ago`;
    } else if (months > 0) {
        return `${months} month${months > 1 ? 's' : ''} ago`;
    } else if (days > 0) {
        return `${days} day${days > 1 ? 's' : ''} ago`;
    } else if (hours > 0) {
        return `${hours} hour${hours > 1 ? 's' : ''} ago`;
    } else if (minutes > 0) {
        return `${minutes} minute${minutes > 1 ? 's' : ''} ago`;
    } else {
        return `${seconds} second${seconds > 1 ? 's' : ''} ago`;
    }
}



const getSinglePostDetails = (request, response) => {
    const { user_id, post_id } = request.query; // Retrieve user_id and post_id from query parameters

    try {
        if (!user_id || !post_id) {
            return response.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        // check if user exists
        const userCheckQuery = "SELECT user_id,active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
        connection.query(userCheckQuery, [user_id], async (err, userInformation) => {
            if (err) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if (userInformation.length <= 0) {
                return response.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }
            const isActive = userInformation[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }

            // Fetch post details
            const postQuery = "SELECT post_id, post_type, caption, keyword_ids, location, latitude, longitude, createtime FROM post_master WHERE delete_flag = 0 AND post_id = ?";
            connection.query(postQuery, [post_id], async (error, results) => {
                if (error) {
                    return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "1" });
                }
                
                if (results.length > 0) {
                    try {
                        const post_images = await GetPostImages(post_id);
                        const like_status = await likePostStatus(post_id, user_id);
                        const dislike_status = await dislikePostStatus(post_id, user_id);
                        const likeCount = await likePostCount(post_id);
                        const dislikeCount = await dislikePostCount(post_id);
                        const post_time = calculateTimeElapsed(results[0].createtime);

                        const data = {
                            post_id,
                            post_images,
                            like_status,
                            dislike_status,
                            likeCount,
                            dislikeCount,
                            post_time,
                            post_type: results[0].post_type,
                            caption: results[0].caption,
                            keyword_ids: results[0].keyword_ids,
                            location: results[0].location,
                            latitude: results[0].latitude,
                            longitude: results[0].longitude,
                            createtime: results[0].createtime
                        };

                        return response.status(200).json({ success: true, msg: languageMessages.msgDataFound, data });
                    } catch (error) {
                        console.error('Error in getDetails: ', error);
                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "3" });
                    }
                } else {
                    return response.status(500).json({ success: false, msg: languageMessages.msgDataNotFound, key: 'user not create post' });
                }
            });
        });
    } catch (error) {
        console.error('Error in getDetails: ', error);
        return response.status(500).json({ success: false , msg: languageMessages.internalServerError, key: "2" });
    }
}




const  getPostCount = (request, response) => {
    const { user_id,language_id } = request.query;

    try {
        var account_bal = 0;
        var currency = "0";

        const userQuery = "SELECT account_fund_balance, account_currency,active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
        const userVal = [user_id];
        
        connection.query(userQuery, userVal, (userError, userResult) => {
            if (userError) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: userError });
            }
            const isActive = userResult[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }
            if (userResult.length > 0) {
                account_bal = userResult[0].account_fund_balance;
                currency = userResult[0].account_currency != null ? userResult[0].account_currency:'USD'; 

                const userCheckQuery = "SELECT post_like_count, top_post_fee_usd, top_post_fee_yuan, min_word, max_word FROM admin_manage_master";
                
                connection.query(userCheckQuery, (err, Information) => {
                    if (err) {
                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: err });
                    }
                    if (Information.length <= 0) {
                        return response.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
                    }
                    
                    const post_count = Information[0].post_like_count;
                    const top_post_fee_usd = Information[0].top_post_fee_usd;
                    const top_post_fee_yuan = Information[0].top_post_fee_yuan;
                    const min_word = Information[0].min_word;
                    const max_word = Information[0].max_word;

                    var account_bal1 = 0;
                    var account_bal2 = 0;
                    if(account_bal > 0){
                        account_bal1 = account_bal - top_post_fee_usd;
                    }
                    if(account_bal > 0){
                        account_bal2 = account_bal - top_post_fee_yuan;
                    }

                    var msg;
                    var msgNew;
                    var msgNew1;
                    var msg1;
                    var msg1New;
                    var msg1New1;
                   
                    msgNew = "Note: It costs " + top_post_fee_usd + " USD to send this TOP post ,and the remaining account balance will be " + account_bal1.toFixed(2) + " USD after that.";
                    msgNew1 = "Note: It costs " + top_post_fee_yuan + " Chinese Yuan to send this TOP post ,and the remaining account balance will be " + account_bal2.toFixed(2) + " Chinese Yuan after that.";

                    msg = currency === 'USD' ? msgNew : msgNew1;

                    msg1New = 
                    "注：发送此置顶帖子需要花费 " + top_post_fee_usd + " USD，之后账户余额将为 " + account_bal1.toFixed(2) + " USD。";
                    msg1New1 = 
                    "注意：发送此置顶帖子需要花费 " + top_post_fee_yuan + " 中国元，此后账户余额将剩余 " + account_bal2.toFixed(2) + " 中国元.";

                    msg1 = currency === 'USD' ? msg1New : msg1New1;

                    const topPostMessage = [msg, msg1];

                    let account_bal_new;
                    if (language_id == 0) {
                        account_bal_new = currency != 'USD' ? account_bal.toString() + ' ' + 'Chinese Yuan' : account_bal.toString() + ' ' + 'USD';
                    } else {
                        account_bal_new = currency === 'USD' ? account_bal.toString() + ' ' + 'USD' : account_bal.toString() + ' ' + '中国元';
                    }
                    
                    return response.status(200).json({ success: true, msg: languageMessages.msgDataFound, post_count, topPostMessage, min_word, max_word, account_bal_new, currency });
                });
            } else {
                return response.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }
        });
    } catch (error) {
        return response.status(500).json({ success: false, msg: "Internal server error", error: error });
    }
};






function deg2rad(deg) {
    return deg * (Math.PI / 180);
  }
  
  function getDistance(lat1, lon1, lat2, lon2) {
    const R = 6371; // Radius of the Earth in km
    const dLat = deg2rad(lat2 - lat1);
    const dLon = deg2rad(lon2 - lon1);
    const a =
      Math.sin(dLat / 2) * Math.sin(dLat / 2) +
      Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
      Math.sin(dLon / 2) * Math.sin(dLon / 2);
    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    const distance = R * c; // Distance in km
    return distance;
  }
// distance calculation end


// save unsave start


const saveUnsave = async (req,res) => {

    const { user_id,post_id} = req.body;

    try {

        
        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.msg_empty_param, key : 'user_id' });
        }
        if (!post_id) {
            return res.json({ success: false, msg: languageMessages.msg_empty_param, key : 'post_id' });
        }

                 // check user exists or not start
                 const updatetime = getCurrentTimeInTimeZone();

      var sql = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }
        });

        // check user exists or not end

        // select if like start

        
        var select_like = "SELECT save_id FROM save_master WHERE user_id=? and post_id=? ";
        var select_values = [user_id,post_id];
        connection.query(select_like,select_values,async(selectLikeError,SelectLikeResult)=>{
            if(selectLikeError){
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: 1 });
            }
            if(SelectLikeResult.length > 0){
              
                    var deleteLike = "DELETE FROM save_master WHERE user_id = ? and post_id = ?";
                    var deleteValues = [user_id,post_id];
                    connection.query(deleteLike,deleteValues,(deleteLikeError,deleteLikeResult)=>{
                        if(deleteLikeError){
                            return res.status(200).json({ success: false, msg: languageMessages.internalServerError, data: 2 });
                        }
                       

                        return res.status(200).json({ success: true, msg: languageMessages.UnsaveSuccessfully});

                    });
              
                }else{
                    
                // insert like start
                
                var insertLike = "INSERT INTO save_master (`post_id`, `user_id`, `createtime`, `updatetime`) VALUES (?,?,?,?)";
                var insertValues = [post_id,user_id,currentTime,currentTime];

                connection.query(insertLike,insertValues,(InsertError,InsertResult)=>{
                    if(InsertError){
                        return res.status(200).json({ success: false, msg: languageMessages.likeUnsuccessfully,error:InsertError});
                    }

                        return res.status(200).json({ success: true, msg: languageMessages.saveSuccessfully});
                    
                });
                
                // insert like end 

                } 

            });
    
        

        // select if like end


        
    } catch (error) {
        return res.status(200).json({ success: false, msg: languageMessages.internalServerError,key:10});
    }

}

// save unsave end


// get all saved post start


const getAllSavedPost = async (req, res) => {

    const { user_id ,longitude,latitude,language_id} = req.query; 

    try {
       
        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

         // check user exists or not start

      var sql = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }

        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }

        
         

        var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;

            var query = `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != '' and longitude != '' order by distance asc`;



 

            connection.query(query, async (err, rows) => {
                if (err) {
                    console.error('Error executing MySQL query:', err);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
                }
                var localPostArr = [];
                var topPostArr = [];
               
    
                if (rows.length > 0) {
                    for (const row of rows) {
                        const post_images = await GetPostImages(row.post_id);
                        var post_images_cnt = post_images.length;
                        const like_status = await likePostStatus(row.post_id,user_id);
                        const dislike_status = await dislikePostStatus(row.post_id,user_id);
                        const likeCount = await likePostCount(row.post_id);
                        const dislikeCount = await dislikePostCount(row.post_id);
                        const saveUnsaveStatus = await saveUnsaveStatus1(row.post_id,user_id);
                        // const avatar_image = await GetAvatar(row.avatar_id);  
                        const post_time = calculateTimeElapsed(row.createtime);
                        const post_time_chinese = calculateTimeElapsedChinese(row.createtime);
                        if(user_id == row.user_id){
                            var my_post_status = 1;
                        }else{
                            var my_post_status = 0;
                        }
                        let distance = getDistance(row.latitude, row.longitude, latitude, longitude);
                        distance = Math.floor(distance);
                            
                        let keywords_ids =  row.keyword_ids;
                        // let keyword =  row.keyword;
                        if (keywords_ids === "" || keywords_ids === null) {
                            keywords_ids = 'NA';  
                            // keyword = 'NA';       
                        }

                        // get post user detail data start 

                        let user_type = 1;
                        let about = null;
                        let username = 'NA';
                        let avatar_id = 0;
                        let avatar_image = 'NA';
                        fetchUserData(row.user_id,async(error, userDataArray) => {
                            if (error) {
                                console.error('Error fetching user data:', error);
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                            }
                            
                            if(userDataArray != null){
                                user_type = userDataArray.user_type;
                                about = userDataArray.about;
                                username = userDataArray.username;
                                avatar_id = userDataArray.avatar_id;
                                avatar_image = userDataArray.avatar_image;
                            }
                           
                        
                      
                        // get post user detail data end 
                            if(saveUnsaveStatus === 1){
                        // if(row.post_type === 0){

                            // if(home_local_count >= count_local){

                            localPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': (row.caption) ? row.caption : null ,
                                'keyword_ids':keywords_ids,
                                'keyword': row.caption ? await KeywordMatch(user_id,row.caption) : 'NA',
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                'post_time':language_id==0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        // }
                        // }
                        if(row.post_type === 1){

                            // if(home_top_count >= count_top){
                            topPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': row.caption,
                                'keyword_ids':keywords_ids,
                                'keyword': row.caption ? await KeywordMatch(user_id,row.caption) : 'NA',
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                'post_time':language_id==0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        // }

                        
                        }
                    }
                    });
                    }
                }
    
                if(localPostArr.length === 0){
                    var localPostArr = 'NA';
                }
                if(topPostArr.length === 0){
                    var topPostArr = 'NA';
                }
                // if(topPostArr !== 'NA'){
                //     topPostArr.sort((a, b) => a.distance - b.distance);
                // }
                // if(localPostArr !== 'NA'){
                //    localPostArr.sort((a, b) => a.distance - b.distance);
                // }
                 
                
                const record = { success: true, msg: languageMessages.msgDataFound, localPostArr: localPostArr,topPostArr:topPostArr};
                res.json(record);
            });
        
        });

        // check user exists or not end
       

       
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2 });
    }
};

// get all saved post end

// get all local post start

const getAllLocalPost = async (req,res) =>{
    const { user_id ,longitude,latitude,filter_type,language_id,offset} = req.query; 
    try {

        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

       

        const adminCounts = await getAdminSetPostCnt();
        var topPostCnt = adminCounts[0].top_post_show_km;
        var localPostCnt = adminCounts[0].post_show_km;

        // check user exists or not start  

      var sql = "SELECT user_id,active_flag,currect_location_permanent,current_lattitude,current_longitude from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, err:err});
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }

        // var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;

        // var sql = `(((acos(sin((('${information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${information[0].currect_location_permanent === 1 ? information[0].current_longitude :longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;
        var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;

            var query ;
            // const limit = 12;
            var limit = localPostCnt < 12 ? localPostCnt : 12;
            if((offset+limit) > localPostCnt){
                limit = localPostCnt-offset;
            }

          if(offset >= localPostCnt){
            return res.status(200).json({ success: true, msg: languageMessages.msgDataFound,  localPostArr: 'NA',loadmore:false,topPostCnt,localPostCnt});
          }
            // if(filter_type == 0){
            //     query= `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != ''  AND longitude != '' order by post_id desc LIMIT  ${limit+1} OFFSET  ${offset}`;
            // }
            // else{
                query = `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != ''  AND longitude != '' order by distance asc,post_id desc LIMIT  ${limit+1} OFFSET  ${offset}`;
            // }
            let loadmore = false;
            connection.query(query, async (err, rows) => {
                if (err) {
                    console.error('Error executing MySQL query:', err);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
                }
                var localPostArr = [];    
                if (rows.length > 0) {
                    for (const row of rows) {
                        // if(row.post_type === 0){

                        const post_images = await GetPostImages(row.post_id);
                        var post_images_cnt = post_images.length;
                        const like_status = await likePostStatus(row.post_id,user_id);
                        const dislike_status = await dislikePostStatus(row.post_id,user_id);
                        const likeCount = await likePostCount(row.post_id);
                        const dislikeCount = await dislikePostCount(row.post_id);
                        const saveUnsaveStatus = await saveUnsaveStatus1(row.post_id,user_id);
                        const post_time = calculateTimeElapsed(row.createtime);
                        const post_time_chinese = calculateTimeElapsedChinese(row.createtime);
                        if(user_id == row.user_id){
                            var my_post_status = 1;
                        }else{
                            var my_post_status = 0;
                        }
                        // let distance = getDistance(row.latitude, row.longitude, latitude, longitude);
                        
                        // let distance = getDistance(row.latitude, row.longitude, information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude, information[0].currect_location_permanent === 1 ? information[0].current_longitude : longitude);
                        let distance = getDistance(row.latitude, row.longitude,latitude,longitude);
                        distance = Math.floor(distance);
                            
                        let keywords_ids =  row.keyword_ids;
                        let keyword =  row.keyword;
                        if (keywords_ids === "" || keywords_ids === null || keyword == null) {
                            keywords_ids = 'NA';  
                            keyword = 'NA';       
                        }

                        let user_type = 1;
                        let about = null;
                        let username = 'NA';
                        let avatar_id = 0;
                        let avatar_image = 'NA';
                        fetchUserData(row.user_id,async(error, userDataArray) => {
                            if (error) {
                                console.error('Error fetching user data:', error);
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                            }
                            if(userDataArray != null){
                                user_type = userDataArray.user_type;
                                about = userDataArray.about;
                                username = userDataArray.username;
                                avatar_id = userDataArray.avatar_id;
                                avatar_image = userDataArray.avatar_image;
                            }
                      
                        // get post user detail data end 

                        let caption = row.caption != "" ? row.caption:null;


                            localPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': (caption) ? caption : null ,
                                'keyword_ids':keywords_ids,
                                'keyword': caption ? await KeywordMatch(user_id,caption) : 'NA',
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                // 'post_time':post_time,
                                'post_time':language_id==0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        // }
                    });
                    // }
                }
    
                if(localPostArr.length === 0){
                    var localPostArr = 'NA';
                }
                if(localPostArr.length == limit-1){
                    loadmore = true;
                }
           
               
                 
            }

        //    if (filter_type == 0 && localPostArr != 'NA') {
        //         localPostArr.sort((a, b) => {
        //             const dateA = new Date(a.createtime);
        //             const dateB = new Date(b.createtime);
        //             return dateB - dateA;
        //         });
        //     }
            
                const record = { success: true, msg: languageMessages.msgDataFound, localPostArr: localPostArr,loadmore:loadmore,currentTime};
                res.json(record);
            });
            
        
        });

        // check user exists or not end
       

       
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2 });
    }
}
// get all local post end



// get all top post start

const getAllTopPost = async (req,res) =>{
    const { user_id ,longitude,latitude,filter_type,language_id, offset } = req.query; 

    try {
       
        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }


        const adminCounts = await getAdminSetPostCnt();
        var topPostCnt = adminCounts[0].top_post_show_km;
        var localPostCnt = adminCounts[0].post_show_km;


         // check user exists or not start

      var sql = "SELECT user_id,active_flag,currect_location_permanent,current_lattitude,current_longitude from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, err:err});
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }
        
          
            // information[0].currect_location_permanent === 1 ? 

        // var sql = `(((acos(sin((('${information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${information[0].currect_location_permanent === 1 ? information[0].current_longitude :longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;
        var sql = `(((acos(sin((('${latitude}')*pi()/180)) * sin((latitude*pi()/180))+cos(('${latitude}'*pi()/180)) * cos((latitude*pi()/180)) * cos((('${longitude}'-longitude)*pi()/180))))*180/pi())*60*1.1515*1.609344) as distance`;

            var query ;
            var limit = topPostCnt < 12 ? topPostCnt : 12;
            if((offset+limit) > topPostCnt){
                limit = topPostCnt-offset;
            }
          if(offset >= topPostCnt){
            return res.status(200).json({ success: true, msg: languageMessages.msgDataFound,  topPostArr: 'NA',loadmore:false,topPostCnt,localPostCnt});
          }
        
            // if(filter_type == 0){
                // query= `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != '' and longitude != '' AND post_type = 1 order by post_id desc LIMIT  ${limit} OFFSET  ${offset}`;

                // query = `SELECT user_id, post_id, post_type, caption, keyword_ids, location, latitude, longitude, createtime, keyword, ${sql}
                // FROM post_master as ps
                // WHERE delete_flag = 0 
                // AND latitude != '' 
                // AND longitude != '' 
                // AND post_type = 1 
                // ORDER BY post_id DESC
                // LIMIT ${limit+1}
                // OFFSET ${offset}`;
                            
            // }else{
                query = `SELECT user_id,post_id,post_type,caption,keyword_ids,location,latitude,longitude,createtime,keyword,${sql} FROM post_master as ps  WHERE delete_flag = 0 and latitude != '' and longitude != ''  AND post_type = 1 order by distance asc, post_id desc LIMIT  ${limit+1} OFFSET  ${offset}`;

                // query = `SELECT user_id, post_id, post_type, caption, keyword_ids, location, latitude, longitude, createtime, keyword, ${sql}
                // FROM post_master as ps
                // WHERE delete_flag = 0 
                // AND latitude != '' 
                // AND longitude != '' 
                // AND post_type = 1 
                // ORDER BY distance ASC
                // LIMIT ${limit+1}
                // OFFSET ${offset}`;
            // }

            let loadmore = false;
            connection.query(query, async (err, rows) => {
                if (err) {
                    console.error('Error executing MySQL query:', err);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
                }
                var topPostArr = [];
              
                var post_images_cnt = 0;
                if (rows.length > 0) {
                    for (const row of rows) {
                        const post_images = await GetPostImages(row.post_id);
                        // post_images_cnt = (post_images.length > 0) ?  post_images.length : 0 ;
                                  var post_images_cnt = post_images.length;
                        const like_status = await likePostStatus(row.post_id,user_id);
                        const dislike_status = await dislikePostStatus(row.post_id,user_id);
                        const likeCount = await likePostCount(row.post_id);
                        const dislikeCount = await dislikePostCount(row.post_id);
                        const saveUnsaveStatus = await saveUnsaveStatus1(row.post_id,user_id);
                        // const avatar_image = await GetAvatar(row.avatar_id);  
                        const post_time = calculateTimeElapsed(row.createtime);
                        const post_time_chinese = calculateTimeElapsedChinese(row.createtime);
                        if(user_id == row.user_id){
                            var my_post_status = 1;
                        }else{
                            var my_post_status = 0;
                        }


                        // let distance = getDistance(row.latitude, row.longitude, information[0].currect_location_permanent === 1 ? information[0].current_lattitude : latitude, information[0].currect_location_permanent === 1 ? information[0].current_longitude : longitude);
                        let distance = getDistance(row.latitude, row.longitude,latitude,longitude);
                        distance = Math.floor(distance);
                            
                        let keywords_ids =  row.keyword_ids;
                        let keyword =  row.keyword;
                        if (keywords_ids === "" || keywords_ids === null || keyword == null) {
                            keywords_ids = 'NA';  
                            keyword = 'NA';       
                        }

                        // get post user detail data start 

                        let user_type = 1;
                        let about = null;
                        let username = 'NA';
                        let avatar_id = 0;
                        let avatar_image = 'NA';
                        fetchUserData(row.user_id,async(error, userDataArray) => {
                            if (error) {
                                console.error('Error fetching user data:', error);
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                            }


                            if(userDataArray != null){
                            user_type = userDataArray.user_type;
                            about = userDataArray.about;
                            username = userDataArray.username;
                            avatar_id = userDataArray.avatar_id;
                            avatar_image = userDataArray.avatar_image;
                            }
                      
                        // get post user detail data end 

                        let caption = row.caption != "" ? row.caption:null;
                

                    
                            topPostArr.push({
                                'my_post_status': my_post_status,
                                'post_id': row.post_id,
                                'other_user_id': row.user_id,
                                'user_type_label': '0=admin,1=guest,2=vip',
                                'user_type': user_type,
                                'about': about == "" ? null : about,
                                'username': username,
                                'avatar_id': avatar_id,
                                'avatar_image': avatar_image == null ? 'NA' : avatar_image,
                                'post_image': post_images,
                                'post_type': row.post_type,
                                'caption': (caption) ? caption : null ,
                                'keyword_ids':keywords_ids,
                                'keyword': caption ? await KeywordMatch(user_id,caption) : 'NA',
                                'like_status': like_status,
                                'dislike_status': dislike_status,
                                'likeCount': likeCount,
                                'dislikeCount': dislikeCount,
                                'location': row.location,
                                'latitude': row.latitude,
                                'longitude': row.longitude,
                                'createtime': row.createtime,
                                'createtime_format': formatDate(row.createtime),
                                // 'post_time':post_time,
                                'post_time':language_id == 0? post_time:post_time_chinese,
                                "km_miles" : row.km_miles,
                                "distance" : distance ? distance : 1,
                                "post_images_cnt" : post_images_cnt,
                                "saveUnsaveStatus" : saveUnsaveStatus,
                            });
                        
                    });
                  
                    }
                }
    
        
                if(topPostArr.length === 0){
                    var topPostArr = 'NA';
                }
                // let loadmore = false;
                if(topPostArr.length == limit-1){
                    loadmore = true;
                }
                 
                
       
                //   if (filter_type == 0 && topPostArr != 'NA') {
                //     topPostArr.sort((a, b) => {
                //         const dateA = new Date(a.createtime);
                //         const dateB = new Date(b.createtime);
                //         return dateB - dateA;
                //     });
                // }
  
           
  
                return res.status(200).json({ success: true, msg: languageMessages.msgDataFound,  topPostArr:topPostArr,loadmore:loadmore,topPostCnt,localPostCnt});
            });
        
        });

        // check user exists or not end
       

       
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2 });
    }
}

// keyword match start


function KeywordMatch(user_id, caption) {
    return new Promise((resolve, reject) => {
        const query = `SELECT keyword FROM keyword_master WHERE user_id = ? AND delete_flag = 0`;
        connection.query(query, [user_id], (error, results) => {
            if (error) {
                reject(`Error fetching keywords: ${error.message}`);
                return;
            }

            const matchedKeywordsSet = new Set(); 

            if (results.length > 0) {
                results.forEach(result => {
                    let keyword = result.keyword;
                   
                    keyword = keyword.replace(/^#+|\.+/g, '');

                   
                    const isEnglishKeyword = /^[a-zA-Z0-9]+$/.test(keyword);
                    const isChineseKeyword = /^[\u4E00-\u9FFF]+$/.test(keyword);

                    
                    keyword = keyword.replace(/\./g, ''); 

                    
                    const captionWithoutDots = caption.replace(/\./g, '');

                  
                    const wordsInCaption = captionWithoutDots.split(/\s+/);

                    wordsInCaption.forEach(captionWord => {
                      
                        captionWord = captionWord.replace(/\./g, '');

                        if (captionWord.trim() === '') return; 

                        if (isEnglishKeyword) {
                           
                            if (captionWord.toLowerCase() === keyword.toLowerCase()) {
                                matchedKeywordsSet.add('#' + keyword);
                            }
                        } else if (isChineseKeyword) {
                           
                            if (captionWord.includes(keyword)) {
                                matchedKeywordsSet.add('#' + keyword);
                            }
                        }
                    });
                });

                const matchedKeywords = Array.from(matchedKeywordsSet); // Convert Set to Array

                if (matchedKeywords.length > 0) {
                    resolve(matchedKeywords.join(', '));
                } else {
                    resolve('NA');
                }
            } else {
                resolve('NA');
            }
        });
    });
}


// keyword match end

async function getAdminSetPostCnt() {
    const query = "SELECT post_show_km, top_post_show_km FROM admin_manage_master WHERE delete_flag=0";
    return new Promise((resolve, reject) => {
        connection.query(query, (error, results) => {
            if (error) {
                reject(error);
            } else {
                resolve(results);
            }
        });
    });
}


// check keywords are match or not in caption end

module.exports = {
    CreatePost,
    updatePost,
    upload,
    GetMyPost,
    postLike,
    postDislike,
    DeletePost,
    GetAllPost,
    getSinglePostDetails,
    getPostCount,
    saveUnsave,
    getAllSavedPost,
    getAllLocalPost,
    getAllTopPost,
    GetAvatar
}


