const connection = require("./db");
const messages = require("./language_messages");
const moment = require('moment');
const moment_time = require('moment-timezone');

async function getUserDetails(user_id) {
  try {
    const query =
      "SELECT user_id,login_type,login_type_first,user_type,email,password,username,f_name,l_name,name,dob,age,phone_code,mobile,otp,otp_verify,image,gender,address,latitude,zipcode,bio,active_flag,approve_flag,profile_complete,language_id,facebook_id,google_id,twitter_id,instagram_id,apple_id,notification_status,delete_reason,createtime,updatetime,avatar_id, currect_location_permanent,about FROM user_master WHERE user_id = ?";
    return new Promise((resolve, reject) => {
      connection.query(query, [user_id], async(error, rows) => {
        if (error) {
          console.error("Error:", error);
          reject(error);
        } else {
        //    await setAllDetils(rows[0]);
          resolve(rows.length > 0 ? rows[0] : null);
        }
      });
    });
  } catch (error) {
    console.error("Internal Error", error);
    return "NA";
  }
}


// get other_user different way start

async function fetchUserData(userId, callback) {
  const query = `SELECT * FROM user_master WHERE user_id = ? and delete_flag=0`;

  await connection.query(query, [userId], (error, results) => {
    if (error) {
      callback(error, null);
      return;
    }

    // If data found, return specific fields as an array
    if (results.length > 0) {
      const userData = results[0]; // Assuming user_id is unique

      var avatar_id = userData.avatar_id;
      
      GetAvatar(avatar_id, (error, image) => {
        if(error){
      //     console.error("Error fetching avatar:", error);
      // return;
        }
       
        var avatar_image = 'NA';
        if(userData.image){
          avatar_image = userData.image;
        }else if(image){
          avatar_image = image;
        }else{
          avatar_image = 'NA';
        }


        const userDataArray = {
          user_id : userData.user_id,
          login_type :userData.login_type,
          login_type_first:userData.login_type_first,
          user_type:userData.user_type,
          email:userData.email,
          username:userData.username,
          f_name:userData.f_name,
          l_name:userData.l_name,
          name:userData.name,
          dob:userData.dob,
          age:userData.age,
          phone_code:userData.phone_code,
          mobile:userData.mobile,
          otp:userData.otp,
          otp_verify:userData.otp_verify,
          image:userData.image,
          gender:userData.gender,
          address:userData.address,
          latitude:userData.latitude,
          longitude:userData.longitude,
          zipcode:userData.zipcode,
          bio:userData.bio,
          active_flag:userData.active_flag,
          approve_flag:userData.approve_flag,
          profile_complete:userData.profile_complete,
          language_id:userData.language_id,
          facebook_id:userData.facebook_id,
          google_id:userData.google_id,
          twitter_id:userData.twitter_id,
          instagram_id:userData.instagram_id,
          apple_id:userData.apple_id,
          notification_status:userData.notification_status,
          delete_reason:userData.delete_reason,
          createtime:userData.createtime,
          updatetime:userData.updatetime,
          signup_step:userData.signup_step,
          avatar_id:userData.avatar_id,
          // avatar_image: userData.image ? userData.image:avatar_image,
          avatar_image: avatar_image,
          currect_location_permanent:userData.currect_location_permanent,
          about:userData.about,
          km_miles:userData.km_miles
        };
        callback(null, userDataArray);

      });
        
     
    } else {
      // If no data found for the given user_id
      callback(null, null);
    }
  });
}

// get other_user different way end
function GetAvatar(avatar_id, callback){
  const query = `SELECT avatar_image FROM avatar_master WHERE avatar_id = ? and delete_flag=0`;

  connection.query(query, [avatar_id], (error, results) => {
    if (error) {
      callback(error, null);
      return;
    }

    // If data found, return specific fields as an array
    if (results.length > 0) {
      const userData = results[0]; // Assuming user_id is unique
      
      const image = userData.avatar_image; // Assuming avatar_image is the correct field name
             
      callback(null, image);
    } else {
      // If no data found for the given user_id
      callback(null, null);
    }
  });
}





function timeAgo(dateTime) {
  const timestamp = moment(dateTime);
  const currentTime = moment();

  if (currentTime.isAfter(timestamp)) {
    const diff = moment.duration(currentTime.diff(timestamp));
    const durations = [
      { unit: 'seconds', num: 60 },
      { unit: 'minutes', num: 60 },
      { unit: 'hours', num: 24 },
      { unit: 'days', num: 30 },
      { unit: 'months', num: 12 },
      { unit: 'years', num: 10 },
    ];

    for (let i = 0; i < durations.length - 1 && diff.units[durations[i].unit] >= durations[i + 1].num; i++) {
      diff.subtract(durations[i].num, durations[i].unit);
    }

    return diff.as(durations[i].unit) + ' ' + durations[i].unit + ' ago';
  }
}


function getCurrentTimeInTimeZone() {
  const timezone = 'America/New_York';
  return moment_time.tz(timezone).format('YYYY-MM-DD HH:mm:ss');
}

module.exports = {
  getUserDetails,
  fetchUserData,
  timeAgo,
  GetAvatar,
  getCurrentTimeInTimeZone
};
