const crypto = require('crypto');
const fs = require('fs');
const multer = require('multer');
const connection = require('./db');
const compressing = require('compressing');
// const random = require('random');
// Function to generate a random OTP
function generateRandomOTP(length = 6) {
    let otp = '';
    const chars = '123456789';
    for (let i = 0; i < length; i++) {
        otp += chars.charAt(Math.floor(Math.random() * chars.length));
    }
    return otp;
}



function DeviceTokenStore_1_Signal(user_id, device_type, player_id, callback) {

    const inserttime = new Date().toISOString().slice(0, 19).replace('T', ' ');

    // Check if player_id exists
    connection.query('SELECT player_id FROM user_notification WHERE player_id = ?', [player_id], function (error, results, fields) {
        if (error) throw error;

        if (results.length > 0) {
            // Delete all records of this player_id
            connection.query('DELETE FROM user_notification WHERE player_id = ? OR user_id = ?', [player_id, user_id], function (error, results, fields) {
                if (error) throw error;

                if (results.affectedRows <= 0) {
                    // 2nd attempt to delete
                    connection.query('DELETE FROM user_notification WHERE player_id = ? OR user_id = ?', [player_id, user_id], function (error, results, fields) {
                        if (error) throw error;

                        if (results.affectedRows <= 0) {
                            callback("no");
                        } else {
                            // Insert new record
                            connection.query('INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?, ?, ?, ?)', [user_id, device_type, player_id, inserttime], function (error, results, fields) {
                                if (error) throw error;

                                if (results.affectedRows > 0) {
                                    callback("yes");
                                } else {
                                    callback("no");
                                }
                            });
                        }
                    });
                } else {
                    // Insert new record
                    connection.query('INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?, ?, ?, ?)', [user_id, device_type, player_id, inserttime], function (error, results, fields) {
                        if (error) throw error;

                        if (results.affectedRows > 0) {
                            callback("yes");
                        } else {
                            callback("no");
                        }
                    });
                }
            });
        } else {
            // Check if user_id exists
            connection.query('SELECT user_id FROM user_notification WHERE user_id = ?', [user_id], function (error, results, fields) {
                if (error) throw error;

                if (results.length > 0) {
                    // Delete all records of this user_id
                    connection.query('DELETE FROM user_notification WHERE user_id = ?', [user_id], function (error, results, fields) {
                        if (error) throw error;

                        if (results.affectedRows <= 0) {
                            // 2nd attempt to delete
                            connection.query('DELETE FROM user_notification WHERE user_id = ?', [user_id], function (error, results, fields) {
                                if (error) throw error;

                                if (results.affectedRows <= 0) {
                                    callback("no");
                                } else {
                                    // Insert new record
                                    connection.query('INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?, ?, ?, ?)', [user_id, device_type, player_id, inserttime], function (error, results, fields) {
                                        if (error) throw error;

                                        if (results.affectedRows > 0) {
                                            callback("yes");
                                        } else {
                                            callback("no");
                                        }
                                    });
                                }
                            });
                        } else {
                            // Insert new record
                            connection.query('INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?, ?, ?, ?)', [user_id, device_type, player_id, inserttime], function (error, results, fields) {
                                if (error) throw error;

                                if (results.affectedRows > 0) {
                                    callback("yes");
                                } else {
                                    callback("no");
                                }
                            });
                        }
                    });
                } else {
                    // Insert new record
                    connection.query('INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?, ?, ?, ?)', [user_id, device_type, player_id, inserttime], function (error, results, fields) {
                        if (error) throw error;

                        if (results.affectedRows > 0) {
                            callback("yes");
                        } else {
                            callback("no");
                        }
                    });
                }
            });
        }
    });
}




// Function to upload a single file with compression

const uploadSingleFile = async (file, folder_name, file_type) => {

    const storage = multer.diskStorage({

        destination: function (req, file, cb) {

            cb(null, folder_name);

        },

        filename: function (req, file, cb) {

            cb(null, Date.now() + '-' + file.originalname);

        }

    });



    const upload = multer({ storage: storage }).single(file_type);



    return new Promise((resolve, reject) => {

        upload(file, null, async function (err) {

            if (err) {

                reject(err);

            } else {

                const filePath = `${folder_name}/${file.filename}`;

                const compressedFileName = `${Date.now()}_${file.originalname}.zip`;

                const compressedFilePath = `${folder_name}/${compressedFileName}`;

                try {

                    await compressing.zip.compressFile(filePath, compressedFilePath);

                    resolve(compressedFileName);

                } catch (error) {

                    reject(error);

                }

            }

        });

    });

};



// Function to upload multiple files with compression

const uploadMultipleFiles = async (file_arr, folder_name) => {

    const storage = multer.diskStorage({

        destination: function (req, file, cb) {

            cb(null, folder_name);

        },

        filename: function (req, file, cb) {

            cb(null, Date.now() + '-' + file.originalname);

        }

    });



    const upload = multer({ storage: storage }).array('files', file_arr.length);



    return new Promise((resolve, reject) => {

        upload(file_arr, null, async function (err) {

            if (err) {

                reject(err);

            } else {

                const compressedFileNames = [];

                for (const file of file_arr) {

                    const filePath = `${folder_name}/${file.filename}`;

                    const compressedFileName = `${Date.now()}_${file.originalname}.zip`;

                    const compressedFilePath = `${folder_name}/${compressedFileName}`;

                    try {

                        await compressing.zip.compressFile(filePath, compressedFilePath);

                        compressedFileNames.push(compressedFileName);

                    } catch (error) {

                        reject(error);

                    }

                }

                resolve(compressedFileNames);

            }

        });

    });

};



// Function to generate a random name with 6 letters

function generateRandomName() {
    const alphabet = 'abcdefghijklmnopqrstuvwxyz';
    let name = '';
    // Generate 6 random letters
    for (let i = 0; i < 6; i++) {
        const randomIndex = Math.floor(Math.random() * alphabet.length);
        name += alphabet.charAt(randomIndex);
    }
    return name;
}



// cheak user active or not 
async function cheakUseractiveDeactive(user_id){
    try {
        const sql = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
        const result = await new Promise((resolve, reject) => {
            connection.query(sql, [user_id], (err, result) => {
                if (err) {
                    reject(err);
                } else {
                    resolve(result);
                }
            });
        });
        return result;
    } catch (error) {
        console.log("error: ", error);
        response.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
    }
}


module.exports = {
    generateRandomOTP,
    DeviceTokenStore_1_Signal,
    uploadSingleFile,
    uploadMultipleFiles,
    generateRandomName,
    cheakUseractiveDeactive
}