// content.js

const express = require('express');
const app = express();
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');

const GetContent = (req, res) => {
   
    try {

        const {language_id} = req.query;
       
        var query = "SELECT content_id, content_type,content,content_1 FROM content_master WHERE delete_flag = ?";
        connection.query(query, [0], (err, rows) => {
            if (err) {
                console.error('Error executing MySQL query:', err);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0] });
            }
            let content_arr = [];

            var webservice_url = 'https://youngdecade.org/dalaba/api_controller/webservice/';

            if (rows.length > 0) {
                content_arr = rows.map(row => ({
                    content_id: row.content_id,
                    content_type: row.content_type,
                    content: [row.content,row.content_1],
                    content_url: `${webservice_url}get_all_content_url?content_type=${row.content_type}&language_id=${language_id}`
                }));
            } else {
                content_arr = 'NA';
            }
            const record = { success: true, msg: languageMessages.msgDataFound, data: content_arr };
            res.json(record);
        })
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
}



const getAllContentUrl = (request, response) => {
    try {
        const { content_type, language_id } = request.query;

        const query1 = "SELECT content_id, content_type, content, content_1 FROM content_master WHERE delete_flag = 0 AND content_type = ?";
        const val1 = [content_type];
        
        connection.query(query1, val1, (valError, valResult) => {
            if (valError) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
            }

            if (valResult.length === 0) {
                return response.status(404).json({ success: false, msg: languageMessages.msgDataNotFound });
            }
            let content_en = 'NA';
            if(language_id==0){
                content_en = valResult[0].content
            }else{
                content_en = valResult[0].content_1 
            }
           
             
            let new12 = '<html><head><meta charset="utf-8"><meta http-equiv="Content-Security-Policy" content="default-src * data: gap: content:"><meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, minimum-scale=1, user-scalable=no, minimal-ui"><title>Data</title></head><body style="word-break: break-all;">' + content_en + '</body></html>';


            return response.send(new12);
        });
    } catch (error) {
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
};


const GetAllAvatar = (req, res) => {
   
    try {
       
        var query = "SELECT `avatar_id`, `avatar_image` FROM avatar_master WHERE delete_flag = ?";
        connection.query(query, [0], (err, rows) => {
            if (err) {
                console.error('Error executing MySQL query:', err);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
            }
            let content_arr = [];
            
            if (rows.length > 0) {
    content_arr = rows.map(row => {
        var finalhightwidht = getRandomHeight();
        return {
            avatar_id: row.avatar_id,
            avatar_image: row.avatar_image,
            height: Math.floor(finalhightwidht),
            width: Math.floor(finalhightwidht),
            'status': false,
        };
    });
} else {
                content_arr = 'NA';
            }
            const record = { success: true, msg: languageMessages.msgDataFound, data: content_arr };
            res.json(record);
        })
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
}

const GetAdminMail = (req, res) => {
   
    try {
       
        var query = "SELECT `content` FROM content_master WHERE delete_flag = ? and content_id = 18";
        connection.query(query, [0], (err, rows) => {
            if (err) {
                console.error('Error executing MySQL query:', err);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
            }
            let content_arr = [];

            var email = 'NA';
            if (rows.length > 0) {
                // content_arr = rows.map(row => ({
                    var email = rows[0].content
                // }));
            } 
            // else {
            //     content_arr = 'NA';
            // }
            const record = { success: true, msg: languageMessages.msgDataFound, data: email };
            res.json(record);
        })
    } catch (error) {
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
}


function getRandomHeight() {
    // Generate a random number between 0 and 1
    const random = Math.random();
    
    // Scale the random number to fit within the range (10 to 30)
    const minHeight = 10;
    const maxHeight = 30;
    const heightRange = maxHeight - minHeight;
    const scaledRandom = random * heightRange;
    
    // Add the minimum height to the scaled random number to get the final height
    const finalHeight = scaledRandom + minHeight;
    
    // Return the final height
    return finalHeight;
  }




app.get('/get_all_content_url', (req, res) => {
    let content_id = req.query.content_id;
    let content_type = req.query.content_type;
    let user_type = req.query.user_type;
  
    connection.query('SELECT `content_id`, `content_type`, `content`, `content_1`, `delete_flag` FROM `content_master` WHERE delete_flag = 0 AND content_id = ? AND content_type = ?', [content_id, content_type], (error, results, fields) => {
      if (error) {
        console.error('Error fetching content: ' + error);
        res.status(500).send('Internal Server Error');
        return;
      }
  
      if (results.length > 0) {
        let content;
        if (user_type == 1) {
          content = results[0].content;
        } else {
          content = results[0].content_1;
        }
        res.send(content);
      } else {
        res.status(404).send('Content not found');
      }
    });
  });

module.exports = {
    GetContent,
    GetAllAvatar,
    GetAdminMail,
    getAllContentUrl
}
