const express = require('express');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const {cheakUseractiveDeactive} = require('./function_app_common');


const GetAllKeywords = async (req, res) => {
    try {
        const { user_id } = req.query; // Retrieve user_id from query parameters

        if (!user_id) {
            return res.status(400).json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        // Check if user exists
        const userQuery = "SELECT user_id,active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
        connection.query(userQuery, [user_id], async (err, information) => {
            if (err) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if (information.length <= 0) {
                return res.status(404).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }
            const isActive = information[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }

            // User exists, proceed to fetch keywords
            const keywordQuery = "SELECT `keyword_id`, `keyword` FROM keyword_master WHERE delete_flag = 0 AND user_id = ? ORDER BY keyword ASC";
            connection.query(keywordQuery, [user_id], (err, rows) => {
                if (err) {
                    console.error('Error executing MySQL query:', err);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
                }
                let key_arr = [];

                if (rows.length > 0) {
                    key_arr = rows.map(row => ({
                        keyword_id: row.keyword_id,
                        keyword: row.keyword,
                        status: false,
                    }));
                } else {
                    key_arr = 'NA';
                }

                const record = { success: true, msg: languageMessages.msgDataFound, data: key_arr };
                res.json(record);
            });
        });
    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
};

// get all keywords end

// add keyword start


const AddKeywords = (req, res) => {
    try {
        // const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");
       
        const { user_id,keyword} = req.body;

        if (!user_id) {
            return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
        }


         // check user exists or not start

      var sql = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
      var values = [user_id];
      connection.query(sql, values, async (err, information) => {
          if (err) {
              return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
          }
          if(information.length <= 0){
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        }
        const isActive = information[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }
        });
       

        // // delete old keywords start

        // const deleteQuery = "DELETE FROM keyword_master WHERE user_id = ? ";
        // const deleteValue = [user_id];
        // connection.query(deleteQuery,deleteValue,(deleteError,deleteResult)=>{
        //     if(deleteError){
        //         return res.status(500).json({ success: false, msg: languageMessages.internalServerError,key:1 });
        //     }
        // })

        // // delete old keywords end

        const insertPostQuery = "INSERT INTO keyword_master (`user_id`, `createtime`, `updatetime`, `keyword`) VALUES (?,now(), now(),?)";
const keywords = keyword.split(',').map(keyword => keyword.trim()); // Split the comma-separated keywords into an array

// Iterate over each keyword and execute the insertion query
keywords.forEach(keyword => {
    const values = [user_id, keyword];

    connection.query(insertPostQuery, values, (postError, postResults) => {
        if (postError) {
            console.error("Error inserting post:", postError);
            return res.status(500).json({ success: false, msg: languageMessages.KeywordAddedUnsuccessfully });
        }
    });
});

return res.status(200).json({ success: true, msg: languageMessages.KeywordAddedSuccessfully });


    } catch (err) {
        // console.error("Internal server error:", err);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError,key:2});
    }
}

  // delete  keywords start

  const removeKeyword = (req, res) => {
    const { user_id, keyword_id } = req.body;

    if (!user_id || !keyword_id) {
        return res.status(400).json({ success: false, msg: languageMessages.msg_empty_param });
    }

    // Check if user exists
    var userSql = "SELECT user_id,active_flag FROM user_master WHERE delete_flag = 0 AND user_id = ?";
    var userValues = [user_id];
    connection.query(userSql, userValues, async (userErr, userInformation) => {
        if (userErr) {
            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
        }
        if (userInformation.length <= 0) {
            return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
        } 
        const isActive = userInformation[0].active_flag;
        if (isActive === 0) {
            return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
        }
        // Check if keyword exists
        var keywordSql = "SELECT keyword_id FROM keyword_master WHERE delete_flag = 0 AND keyword_id = ?";
        var keywordValues = [keyword_id];
        connection.query(keywordSql, keywordValues, async (keywordErr, keywordInformation) => {
            if (keywordErr) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if (keywordInformation.length <= 0) {
                return res.status(500).json({ success: false, msg: languageMessages.msgKeywordNotFound, data: {} });
            }

            // Delete the keyword
            const deleteQuery = "DELETE FROM keyword_master WHERE user_id = ? AND keyword_id = ?";
            const deleteValues = [user_id, keyword_id];
            connection.query(deleteQuery, deleteValues, (deleteError, deleteResult) => {
                if (deleteError) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1 });
                }
                return res.status(200).json({ success: true, msg: languageMessages.KeywordDeleteSuccessfully });
            });
        });
    });
}

  // delete old keywords end

// add keyword end

module.exports = {
    GetAllKeywords,
    AddKeywords,
    removeKeyword
}