const express = require('express');
const multer = require('multer');
// const router = express.Router();
const connection = require('./db');
const crypto = require('crypto');
const languageMessages = require('./language_messages');
const {DeviceTokenStore_1_Signal,generateRandomName, cheakUseractiveDeactive} = require('./function_app_common');
const {getUserDetails,fetchUserData,getCurrentTimeInTimeZone} = require("./function_app");
const {GetAvatar} = require("./all_post");
const {currentTime} = require("./time");
const moment = require("moment");

const { connected } = require('process');




const login = async (req, res) => {

    try {
        const { user_type, name, password, player_id, device_type, login_type } = req.body;

        if (!user_type || !name || !player_id || !device_type || !login_type) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        const updatetime = getCurrentTimeInTimeZone();

        if (user_type == 2) {
       
            const hashedPassword = md5(password); // Hashing password
            const sql = 'SELECT user_id,password,active_flag FROM user_master WHERE delete_flag = 0 AND username = ? AND password = ? AND user_type = 2';
            connection.query(sql, [name,hashedPassword], async (error, results, fields) => {
                if (error) {
                    console.error("Database Error:", error);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
                }

                if (!results || results.length === 0) {
                    return res.json({ success: false, msg: languageMessages.usernameAndPassword, data: {} });
                }

                const isActive = results[0].active_flag;
                if (isActive === 0) {
                    return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
                }

                const user_id = results[0].user_id;
              
                try {
                    // const userDetails = await getUserDetails(user_id);
                    fetchUserData(user_id, (error, userDataArray) => {
                        if (error) {
                            console.error('Error fetching user data:', error);
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                        }
                        return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDataArray });
                    });
                    
                } catch (getUserDetailsError) {
                    console.error("Error getting user details:", getUserDetailsError);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '3', error: getUserDetailsError });
                }
            });
        } 
        else if(user_type == 1){
            const selectGuest = "SELECT user_id FROM user_master WHERE delete_flag = 0 and username = ? ";
            connection.query(selectGuest, [name], async (selectError, selectResult) => {
                if (selectError) {
                    console.error("Database Error:", selectError);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
                }

                if (!selectResult || selectResult.length === 0) {
                    const insertGuest = "INSERT INTO user_master(username, user_type, createtime, updatetime,profile_complete) VALUES (?, ?,?,?,1)";
                    connection.query(insertGuest, [name, user_type,currentTime,currentTime], (insertError, insertResult) => {
                        if (insertError) {
                            console.error("Database Error:", insertError);
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '4' });
                        }

                        fetchUserData(insertResult.insertId, (error, userDataArray) => {
                            if (error) {
                                console.error('Error fetching user data:', error);
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                            }
                            return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDataArray });
                        });
                    });
                } else {
                    const user_id = selectResult[0].user_id;
                    fetchUserData(user_id, (error, userDataArray) => {
                        if (error) {
                            console.error('Error fetching user data:', error);
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
                        }
                        return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDataArray });
                    });
                }
            });
        }
    } catch (err) {
        console.error("Error updating user:", err);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: err });
    }
}


const GuestLogin = async (req, res) => {
    const { username, user_type, device_type, login_type, player_id } = req.body;
    try {
        if (!user_type || !player_id || !device_type || !login_type) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        if (username) {
            var sql = "SELECT username,user_id from user_master where delete_flag = 0 AND username = ?";
            var values = [username];
            connection.query(sql, values, async (err, information) => {
                if (err) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
                } else {
                    console.log("information : ", information);
                    if (information.length > 0) {
                        const user_id = information[0].user_id;
                        DeviceTokenStore_1_Signal(user_id, device_type,player_id, function(result){ 
                            return result.json({ success: false, msg: languageMessages.internalServerError, data: {} });
                        });
                        try {
                            var userDetails = await getUserDetails(user_id); 
                            return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDetails });
                        } catch (getUserDetailsError) {
                            console.error("Error getting user details:", getUserDetailsError);
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: getUserDetailsError });
                        }
                    } else {
                        console.log("userNotFound");
                        // const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");
                        var sql = "INSERT INTO user_master(username,user_type,login_type,createtime,updatetime) VALUES (?,?,?,?,?)";
                        var values = [username, user_type, login_type,currentTime,currentTime];
                        connection.query(sql, values, (err, info) => {
                            if (err) {
                                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key':'6' });
                            } 
                            var user_id = info.insertId;
                            DeviceTokenStore_1_Signal(user_id, device_type,player_id, function(result){ 
                                return result.json({ success: false, msg: languageMessages.internalServerError, data: {} });
                            });
                            fetchUserData(user_id, (error, userDataArray) => {
                                if (error) {
                                //   console.error('Error fetching user data:', error);
                                  
                                }
                                return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDataArray });
                            })
                           
                        });
                    }
                }
            });
        }else{
            console.log("userNotFound");
            // return res.json({ success: false, msg: "user name not found....." });
            // const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");

            var name;
            if(!username){
                name = generateRandomName();
            }
            var sql = "INSERT INTO user_master(username,user_type,login_type,createtime,updatetime) VALUES (?,?,?,?,?)";
            var values = [name, user_type, login_type, currentTime, currentTime];
            connection.query(sql, values, (err, info) => {
                if (err) {
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key':'7' });
                } 
                const user_id = info.insertId;
                DeviceTokenStore_1_Signal(user_id, device_type,player_id, function(result){ 
                    return result.json({ success: false, msg: languageMessages.internalServerError, data: {} });
                });
                fetchUserData(user_id, (error, userDataArray) => {
                    if (error) {
                      console.error('Error fetching user data:', error);
                      return;
                    }
                    return res.json({ success: true, msg: languageMessages.loginSuccessful, data: userDataArray });
                })
               
            });
        }
    } catch (error) {
        console.log("Error in GuestLogin:", error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: error });
    }
}



const editprofile = async (request, response) => {
    
    const { user_id, username, about, avatar_id, email, password} = request.body;
    

    try {
        if (!user_id) {
            return response.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        const updatetime = getCurrentTimeInTimeZone();

        // Check if user exists
        const userExistQuery = "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
        connection.query(userExistQuery, [user_id], async (err, userInfo) => {
            if (err) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: err });
            }

            if (!userInfo.length) {
                return response.json({ success: false, msg: languageMessages.msgUserNotFound });
            }

            // Check if user is active
            const isActive = userInfo[0].active_flag;
            if (isActive === 0) {
                return response.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }

            // Check if username already exists
            const usernameExistQuery = "SELECT user_id FROM user_master WHERE user_id != ? AND username = ? AND delete_flag = 0";
            connection.query(usernameExistQuery, [user_id, username], async (err, existingUser) => {
                if (err) {
                    return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: err });
                }

                if (existingUser.length > 0) {
                    return response.json({ success: false, msg: languageMessages.userNameAlreadyExists });
                }

                // Prepare update query
                let updateSql = 'UPDATE user_master SET username=?, about=?, avatar_id=?, updatetime=?';
                const values = [username, about, avatar_id,currentTime];

                // Handle optional fields
                if (email && password) {
                    const hashedPassword = md5(password);
                    updateSql += ', email=?, password=?,password_new=?';
                    values.push(email,hashedPassword,password);
                } else if (email) {
                    updateSql += ', email=?';
                    values.push(email);
                } else if (password) {
                    const hashedPassword = md5(password);
                    updateSql += ', password=?,password_new=?';
                    values.push(hashedPassword,password);
                }

                if (request.files && request.files['image']) {
                    const image = request.files['image'];
                    updateSql += ', image=?';
                    values.push(image[0].filename);
                } else if (avatar_id > 0) {
                    // If avatar_id is greater than 0, set image to NULL
                    const avatar_image = await GetAvatar(avatar_id);
                    updateSql += ', image=?';
                    values.push(avatar_image);
                }
                

                updateSql += ' WHERE user_id=? AND delete_flag=0';
                values.push(user_id);

                // Execute update query
                connection.query(updateSql, values, (updateErr, result) => {
                    if (updateErr) {
                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: updateErr });
                    }

                    // Fetch updated user data
                    // const userDataQuery = 'SELECT avatar_id, username, email FROM user_master WHERE user_id=? AND delete_flag=0';
                    fetchUserData(user_id, (fetchErr, userDataArray) => {
                        if (fetchErr) {
                            return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: fetchErr });
                        }
                        return response.json({ success: true, msg: languageMessages.profileUpdateSuccessfully, data: userDataArray });
                    });
                });
            });
        });
    } catch (error) {
        console.log("Error in editProfile: ", error);
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, error: error });
    }
};




const getProfile = async (req, res) => {
    try {
        const { user_id } = req.query; // Retrieve user_id from query parameters

        if (!user_id) {
            return res.json({ success: false, msg: languageMessages.allFieldsRequired, data: {} });
        }

        var sql1 = "SELECT user_id,active_flag from user_master where delete_flag = 0 AND user_id = ?";
        var values = [user_id];
        connection.query(sql1, values, async (err, information) => {
            if (err) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
            }
            if(information.length <= 0){
              return res.status(500).json({ success: false, msg: languageMessages.msgUserNotFound, data: {} });
            }

            const isActive = information[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }
              
          });
          fetchUserData(user_id, (error, userDataArray) => {
            if (error) {
                console.error('Error fetching user data:', error);
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 1});
            }
            const record = { success: true, msg: languageMessages.msgDataFound, data: userDataArray };
            return res.json(record);
        });
         


    } catch (error) {
        console.error('Error in GetAllKeywords:', error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, key: 2});
    }
};


// get profile end

// delete account start

const deleteAccount = (req,res)=>{
    const {user_id,reason} = req.query;
    try {


        var sql = "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag=0";
        connection.query(sql, [user_id], async (errNwe, info) => {
            if (errNwe) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError,errNwe});
            }
            if (info.length <= 0) {
                return res.status(404).json({ success: false, msg: languageMessages.msgUserNotFound });
            }
           const isActive = info[0].active_flag;
            if (isActive === 0) {
                return res.json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
            }
        });

        console.log("user_id : ",user_id);
        const updatetime = getCurrentTimeInTimeZone();
        connection.query('update user_master set delete_flag = 1,updatetime=?,delete_reason=? where user_id=?',[currentTime,reason,user_id],(err,result)=>{
            if (err) {
                return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key':'7' });
            } else{
                
                    return res.status(200).json({ success: true, msg: languageMessages.deleteAccount});
                
            }
        });
    } catch (error) {
        console.log("Error in delete account:", error);
        return res.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: error });
  
    }
}

// delete account end

function md5(input) {
    console.log(input);
    return crypto.createHash('md5').update(input).digest('hex');
}

const randomName = (request, response) => {
    const characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    let name = '';
    // Generate 6 random characters (letters or numbers)
    for (let i = 0; i < 6; i++) {
        const randomIndex = Math.floor(Math.random() * characters.length);
        name += characters.charAt(randomIndex);
    }
    
    if (name) {
        return response.status(200).json({ success: true, Random_Name: name });
    } else {
        return response.status(200).json({ success: false, Random_Name: "" });
    }
}


const contectUs = (request, response) => {
    const { user_id, name, email, message } = request.body;
    try {
      
        const updatetime = getCurrentTimeInTimeZone();
        
          // check user exists or not end
        var sql = "INSERT INTO contact_us_master(user_id, name, email, message, createtime, updatetime) VALUES (?, ?, ?, ?,?,?)";
        connection.query(sql, [user_id, name, email, message,currentTime,currentTime], (err, result) => {
            if (err) {
                console.error("Error in SQL query:", err);
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: err });
            } else {
                if (result.affectedRows > 0) {
                    return response.status(200).json({ success: true, msg: languageMessages.contactUsSuccessfully});
                } else {
                    return response.status(400).json({ success: false, msg: languageMessages.contactUsUnsuccessfully});
                }
            }
        });
    } catch (error) {
        console.error("Error in contectUs:", error);
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, 'key': '2', error: error });
    }
}


module.exports = {
    login,
    GuestLogin,
    editprofile,
    getProfile,
    deleteAccount,
    randomName,
    contectUs
};

